/**********************************************************************
 *
 * Copyright(c) 2008 Imagination Technologies Ltd. All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope it will be useful but, except 
 * as otherwise stated in writing, without any warranty; without even the 
 * implied warranty of merchantability or fitness for a particular purpose. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 * 
 * The full GNU General Public License is included in this distribution in
 * the file called "COPYING".
 *
 * Contact Information:
 * Imagination Technologies Ltd. <gpl-support@imgtec.com>
 * Home Park Estate, Kings Langley, Herts, WD4 8LZ, UK 
 *
 ******************************************************************************/

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <asm/uaccess.h>
#include <asm/io.h>

#if defined(LMA)
#include <linux/pci.h>
#else
#include <linux/dma-mapping.h>
#endif

#include "bufferclass_example.h"
#include "bufferclass_example_linux.h"

#define DEVNAME	"bc_example"

MODULE_AUTHOR("Imagination Technologies Ltd. <gpl-support@imgtec.com>");
MODULE_LICENSE("GPL");
MODULE_SUPPORTED_DEVICE(DEVNAME);

int BC_Example_Bridge(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg);
int FillBuffer(unsigned int ui32BufferIndex);
int GetBufferCount(unsigned int *pui32BufferCount);

static int AssignedMajorNumber;

static struct file_operations bufferclass_example_fops = {
	ioctl:BC_Example_Bridge,
};


#define unref__ __attribute__ ((unused))

#if defined(LMA)
#define PVR_BUFFERCLASS_MEMOFFSET (220 * 1024 * 1024) 
#define PVR_BUFFERCLASS_MEMSIZE	  (4 * 1024 * 1024)	  

unsigned int g_ui32MemBase = 0;
unsigned int g_ui32MemCurrent = 0;

typedef struct  
{
	union
	{
		IMG_UINT8	aui8PCISpace[256];
		IMG_UINT16	aui16PCISpace[128];
		IMG_UINT32	aui32PCISpace[64];

		struct  
		{
			IMG_UINT16	ui16VenID;
			IMG_UINT16	ui16DevID;
			IMG_UINT16	ui16PCICmd;
			IMG_UINT16	ui16PCIStatus;
		}s;
	}u;
} PCICONFIG_SPACE, *PPCICONFIG_SPACE;

#define VENDOR_ID_PVR					0x1010
#define DEVICE_ID_PVR					0x1CF1

#define PCI_BASEREG_OFFSET_DWORDS		4

#define PVR_MEM_PCI_BASENUM			2
#define PVR_MEM_PCI_OFFSET		    (PVR_MEM_PCI_BASENUM + PCI_BASEREG_OFFSET_DWORDS)



IMG_UINT32 PCIReadDword(IMG_UINT32 ui32Bus, IMG_UINT32 ui32Dev, IMG_UINT32 ui32Func, IMG_UINT32 ui32Reg)
{
    struct pci_dev *dev;
    IMG_UINT32 ui32Value;

    dev = pci_find_slot(ui32Bus, PCI_DEVFN(ui32Dev, ui32Func));
    if (dev)
    {
            pci_read_config_dword(dev, (int) ui32Reg, (u32 *) & ui32Value);
            return (ui32Value);
    }
    else
{
            return (0);
    }
}

IMG_VOID PCIWriteDword(IMG_UINT32 ui32Bus, IMG_UINT32 ui32Dev, IMG_UINT32 ui32Func, IMG_UINT32 ui32Reg, IMG_UINT32 ui32Value)
	{
    struct pci_dev *dev;

    dev = pci_find_slot(ui32Bus, PCI_DEVFN(ui32Dev, ui32Func));
    if (dev)
    {
            pci_write_config_dword(dev, (int) ui32Reg, (u32) ui32Value);
    }
	}

static IMG_UINT32 FindPCIDevice(IMG_UINT16 ui16VenID, IMG_UINT16 ui16DevID, PCICONFIG_SPACE *psPCISpace)
	{
	IMG_UINT32 ui32BusNum;
	IMG_UINT32 ui32DevNum;
	IMG_UINT32 ui32VenDevID;

	
	for (ui32BusNum=0; ui32BusNum < 255; ui32BusNum++)
	{
		
		for (ui32DevNum=0; ui32DevNum < 32; ui32DevNum++)
		{
			
			ui32VenDevID=PCIReadDword(ui32BusNum, ui32DevNum, 0, 0);			

			
			if (ui32VenDevID == (IMG_UINT32)((ui16DevID<<16)+ui16VenID))
			{
				IMG_UINT32 ui32Idx;

				
				PCIWriteDword(ui32BusNum, ui32DevNum, 0, 4, PCIReadDword(ui32BusNum, ui32DevNum, 0, 4) | 0x02);

				
				for (ui32Idx=0; ui32Idx < 64; ui32Idx++)
				{
					psPCISpace->u.aui32PCISpace[ui32Idx] = PCIReadDword(ui32BusNum, ui32DevNum, 0, ui32Idx*4);
				}
				return PVRSRV_OK;
			}
							
		}

	}

	return PVRSRV_ERROR_GENERIC;
	}
#endif



static int __init BC_Example_ModInit(void)
{
#if defined(LMA)
	PCICONFIG_SPACE	sPCISpace;
#endif
	
	AssignedMajorNumber = register_chrdev(0, DEVNAME, &bufferclass_example_fops);

	if (AssignedMajorNumber <= 0)
	{
		printk("BC_Example_ModInit: unable to get major number\n");

		return -EBUSY;
	}

	printk("BC_Example_ModInit: major device %d\n", AssignedMajorNumber);

#if defined(LMA)
	if(FindPCIDevice(VENDOR_ID_PVR, DEVICE_ID_PVR, &sPCISpace) != PVRSRV_OK)
	{
		printk ("BC_Example_ModInit: can't init device (can't find PCI) \n");
		unregister_chrdev(AssignedMajorNumber, DEVNAME);
		return PVRSRV_ERROR_INVALID_DEVICE;
	}
	
	g_ui32MemBase = sPCISpace.u.aui32PCISpace[PVR_MEM_PCI_OFFSET] + PVR_BUFFERCLASS_MEMOFFSET;
#endif

	if(BC_Example_Init() != PVRSRV_OK)
	{
		printk ("BC_Example_ModInit: can't init device\n");
		unregister_chrdev(AssignedMajorNumber, DEVNAME);
		return -ENODEV;
	}

	return 0;
} 

static void __exit BC_Example_ModCleanup(void)
{    
	unregister_chrdev(AssignedMajorNumber, DEVNAME);
	
	if(BC_Example_Deinit() != PVRSRV_OK)
	{
		printk ("BC_Example_ModCleanup: can't deinit device\n");
	}

} 


IMG_VOID *AllocKernelMem(IMG_UINT32 ui32Size)
{
	return kmalloc(ui32Size, GFP_KERNEL);
}

IMG_VOID FreeKernelMem(IMG_VOID *pvMem)
{
	kfree(pvMem);
}

PVRSRV_ERROR AllocContigMemory(	IMG_UINT32 ui32Size,
								IMG_HANDLE unref__ *phMemHandle, 
								IMG_CPU_VIRTADDR *pLinAddr, 
								IMG_CPU_PHYADDR *pPhysAddr)
{
#if defined(LMA)
	IMG_VOID *pvLinAddr;
	
	
	if(g_ui32MemCurrent + ui32Size >= PVR_BUFFERCLASS_MEMSIZE)
	{
		return PVRSRV_ERROR_OUT_OF_MEMORY;
	}

	pvLinAddr = ioremap(g_ui32MemBase + g_ui32MemCurrent, ui32Size);

	if(pvLinAddr)
	{
		pPhysAddr->uiAddr = g_ui32MemBase + g_ui32MemCurrent;
		*pLinAddr = pvLinAddr;	

		
		g_ui32MemCurrent += ui32Size;
		return PVRSRV_OK;
	}
	return PVRSRV_ERROR_OUT_OF_MEMORY;
#else
	dma_addr_t dma;
	IMG_VOID *pvLinAddr;
	
	pvLinAddr = dma_alloc_coherent(NULL, ui32Size, &dma, GFP_KERNEL);

	if(pvLinAddr == IMG_NULL)
	{
		return PVRSRV_ERROR_OUT_OF_MEMORY;
	}
	else
	{
		IMG_VOID *pvPage;
		IMG_VOID *pvEnd = pvLinAddr + ui32Size;

		for(pvPage = pvLinAddr; pvPage < pvEnd;  pvPage += PAGE_SIZE)
		{
			SetPageReserved(virt_to_page(pvPage));
		}

	pPhysAddr->uiAddr = dma;
	*pLinAddr = pvLinAddr;

	return PVRSRV_OK;
	}
	return PVRSRV_ERROR_OUT_OF_MEMORY;
#endif
}

void FreeContigMemory(  IMG_UINT32 ui32Size,
					    IMG_HANDLE unref__ hMemHandle, 
						IMG_CPU_VIRTADDR LinAddr, 
						IMG_CPU_PHYADDR PhysAddr)
{
#if defined(LMA)
	g_ui32MemCurrent -= ui32Size;
	iounmap(LinAddr);
#else
	dma_free_coherent(NULL, ui32Size, LinAddr, (dma_addr_t)PhysAddr.uiAddr);
#endif
}

IMG_SYS_PHYADDR CpuPAddrToSysPAddr(IMG_CPU_PHYADDR cpu_paddr)
{
	IMG_SYS_PHYADDR sys_paddr;
	
	
	sys_paddr.uiAddr = cpu_paddr.uiAddr;
	return sys_paddr;
}

IMG_CPU_PHYADDR SysPAddrToCpuPAddr(IMG_SYS_PHYADDR sys_paddr)
{
	
	IMG_CPU_PHYADDR cpu_paddr;
	
	cpu_paddr.uiAddr = sys_paddr.uiAddr;
	return cpu_paddr;
}

PVRSRV_ERROR OpenPVRServices (IMG_HANDLE *phPVRServices)
{
	
	*phPVRServices = 0;
	return PVRSRV_OK;
}


PVRSRV_ERROR ClosePVRServices (IMG_HANDLE unref__ hPVRServices)
{
	
	return PVRSRV_OK;
}

PVRSRV_ERROR GetLibFuncAddr (IMG_HANDLE unref__ hExtDrv, IMG_CHAR *szFunctionName, PFN_BC_GET_PVRJTABLE *ppfnFuncTable)
{
	if(strcmp("PVRGetBufferClassJTable", szFunctionName) != 0)
		return PVRSRV_ERROR_INVALID_PARAMS;

	
	*ppfnFuncTable = PVRGetBufferClassJTable;

	return PVRSRV_OK;
}


int BC_Example_Bridge(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
{
	int err = -EFAULT;
	int command = _IOC_NR(cmd);
	BC_Example_ioctl_package *psBridge = (BC_Example_ioctl_package *)arg;

	if(!access_ok(VERIFY_WRITE, psBridge, sizeof(BC_Example_ioctl_package)))
		return err;

	switch(command)
	{
		case _IOC_NR(BC_Example_ioctl_fill_buffer):
		{
			if(FillBuffer(psBridge->inputparam) == -1)
				return err;
			break;
		}
		case _IOC_NR(BC_Example_ioctl_get_buffer_count):
		{	
			if(GetBufferCount(&psBridge->outputparam) == -1)
				return err;
			
			break;
		}
		default:
			return err;
	}

	return 0;
}


module_init(BC_Example_ModInit);
module_exit(BC_Example_ModCleanup);

